@@ -6,7 +6,9 @@
  * Date        : 2006-09-13
  * Description : dcraw settings widgets
  *
- * Copyright (C) 2006-2007 by Gilles Caulier <caulier dot gilles at gmail dot com> 
+ * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2007-2008 by Guillaume Castagnino <casta at xwing dot info>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -23,11 +25,16 @@
 #ifndef DCRAW_SETTINGS_WIDGET_H
 #define DCRAW_SETTINGS_WIDGET_H
 
+// Qt includes.
+
+#include <qstring.h>
+#include <qtoolbox.h>
+
 // KDE includes.
 
-#include <ktabwidget.h>
+#include <kurlrequester.h>
 
-// Local Includes.
+// Local includes.
 
 #include "libkdcraw_export.h"
 #include "rawdecodingsettings.h"
@@ -37,77 +44,115 @@ namespace KDcrawIface
 
 class DcrawSettingsWidgetPriv;
 
-class LIBKDCRAW_EXPORT DcrawSettingsWidget : public KTabWidget
+class LIBKDCRAW_EXPORT DcrawSettingsWidget : public QToolBox
 {
     Q_OBJECT
-    
+
 public:
 
-    DcrawSettingsWidget(QWidget *parent, 
-                        bool sixteenBitsOption=false, 
-                        bool outputColorSpaceOption=true, 
-                        bool showAdvancedOptions=false);
+    enum AdvancedSettingsOptions
+    {
+        SIXTEENBITS      = 0x00000001,
+        COLORSPACE       = 0x00000002,
+        POSTPROCESSING   = 0x00000004,
+        BLACKWHITEPOINTS = 0x00000008
+    };
+
+    enum SettingsTabs
+    {
+        DEMOSAICING = 0,
+        WHITEBALANCE,
+        CORRECTIONS,
+        COLORMANAGEMENT
+    };
+
+public:
+
+    DcrawSettingsWidget(QWidget *parent, int advSettings=COLORSPACE);
+    DcrawSettingsWidget(QWidget *parent,
+                        bool sixteenBitsOption=false,
+                        bool outputColorSpaceOption=true,
+                        bool postProcessingOptions=false);
     virtual ~DcrawSettingsWidget();
 
+    void setup(int advSettings);
+
     bool   sixteenBits();
-    bool   useCameraWB();
-    bool   useAutoColorBalance();
+    int    customWhiteBalance();
+    double customWhiteBalanceGreen();
     bool   useFourColor();
     bool   useDontStretchPixels();
     bool   useNoiseReduction();
     bool   useCACorrection();
     bool   useBlackPoint();
+    bool   useWhitePoint();
     int    unclipColor();
     double brightness();
     int    blackPoint();
+    int    whitePoint();
     int    NRThreshold();
     double caRedMultiplier();
     double caBlueMultiplier();
-    bool   useColorMultipliers();
-    double colorMultiplier1();
-    double colorMultiplier2();
-    double colorMultiplier3();
-    double colorMultiplier4();
+    int    medianFilterPasses();
 
+    RawDecodingSettings::WhiteBalance     whiteBalance();
     RawDecodingSettings::DecodingQuality  quality();
+    RawDecodingSettings::InputColorSpace  inputColorSpace();
     RawDecodingSettings::OutputColorSpace outputColorSpace();
 
+    QString inputColorProfile();
+    QString outputColorProfile();
+
     void   setSixteenBits(bool b);
     void   setCameraWB(bool b);
-    void   setAutoColorBalance(bool b);
+    void   setWhiteBalance(RawDecodingSettings::WhiteBalance v);
+    void   setCustomWhiteBalance(int v);
+    void   setCustomWhiteBalanceGreen(double v);
     void   setFourColor(bool b);
     void   setDontStretchPixels(bool b);
     void   setNoiseReduction(bool b);
     void   setUseCACorrection(bool b);
     void   setUseBlackPoint(bool b);
+    void   setUseWhitePoint(bool b);
     void   setUnclipColor(int v);
     void   setBrightness(double b);
     void   setBlackPoint(int b);
+    void   setWhitePoint(int b);
     void   setNRThreshold(int b);
     void   setcaRedMultiplier(double b);
     void   setcaBlueMultiplier(double b);
+    void   setMedianFilterPasses(int p);
     void   setQuality(RawDecodingSettings::DecodingQuality q);
+    void   setInputColorSpace(RawDecodingSettings::InputColorSpace c);
     void   setOutputColorSpace(RawDecodingSettings::OutputColorSpace c);
-    void   setUseColorMultipliers(bool b);
-    void   setcolorMultiplier1(double b);
-    void   setcolorMultiplier2(double b);
-    void   setcolorMultiplier3(double b);
-    void   setcolorMultiplier4(double b);
+    void   setInputColorProfile(const QString& path);
+    void   setOutputColorProfile(const QString& path);
 
     void   setDefaultSettings();
 
+    void   setEnabledBrightnessSettings(bool b);
+    bool   brightnessSettingsIsEnabled();
+
+    void   updateMinimumWidth();
+
+    KURLRequester* inputProfileUrlEdit() const;
+    KURLRequester* outputProfileUrlEdit() const;
+
 signals:
 
     void signalSixteenBitsImageToggled(bool);
+    void signalSettingsChanged();
 
 private slots:
 
+    void slotWhiteBalanceToggled(int);
     void slotsixteenBitsImageToggled(bool);
-    void slotColorMultToggled(bool);
     void slotUnclipColorActivated(int);
     void slotNoiseReductionToggled(bool);
     void slotCACorrectionToggled(bool);
     void processDcrawURL(const QString&);
+    void slotInputColorSpaceChanged(int);
+    void slotOutputColorSpaceChanged(int);
 
 private:
 
