@@ -39,7 +39,7 @@ extern "C"
 #include <cstdio>
 #include <cmath>
 
-// Qt Includes.
+// Qt includes.
 
 #include <qfile.h>
 #include <qtimer.h>
@@ -100,7 +100,6 @@ public:
     QTimer              *queryTimer;
 
     KProcess            *process;
-
 };
 
 KDcraw::KDcraw()
@@ -136,9 +135,24 @@ bool KDcraw::loadDcrawPreview(QImage& image, const QString& path)
 }
 
 bool KDcraw::loadEmbeddedPreview(QImage& image, const QString& path)
+{
+    QByteArray imgData;
+
+    if ( loadEmbeddedPreview(imgData, path) )
+    {
+        if (image.loadFromData( imgData ))
+        {
+            qDebug("Using embedded RAW preview extraction");
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool KDcraw::loadEmbeddedPreview(QByteArray& imgData, const QString& path)
 {
     FILE       *f=NULL;
-    QByteArray  imgData;
     const int   MAX_IPC_SIZE = (1024*32);
     char        buffer[MAX_IPC_SIZE];
     QFile       file;
@@ -188,13 +202,7 @@ bool KDcraw::loadEmbeddedPreview(QImage& image, const QString& path)
     pclose( f );
 
     if ( !imgData.isEmpty() )
-    {
-        if (image.loadFromData( imgData ))
-        {
-            qDebug("Using embedded RAW preview extraction");
-            return true;
-        }
-    }
+        return true;
 
     return false;
 }
@@ -569,7 +577,7 @@ bool KDcraw::loadFromDcraw(const QString& filePath, QByteArray &imageData,
                            int &width, int &height, int &rgbmax)
 {
     m_cancel      = false;
-    d->dataPos     = 0;
+    d->dataPos    = 0;
     d->filePath   = filePath;
     d->running    = true;
     d->normalExit = false;
@@ -590,7 +598,7 @@ bool KDcraw::loadFromDcraw(const QString& filePath, QByteArray &imageData,
     // So we make up some progress (0% - 40%), using the file size as an indicator how long it might take.
     QTime dcrawStartTime = QTime::currentTime();
     int fileSize         = QFileInfo(filePath).size();
-    
+
     // This is the magic number that describes how fast the function grows
     // It _should_ be dependent on how fast the computer is, but we do not have this piece of information
     // So this is a number that works well on my computer.
@@ -610,7 +618,7 @@ bool KDcraw::loadFromDcraw(const QString& filePath, QByteArray &imageData,
             int elapsedMsecs = dcrawStartTime.msecsTo(QTime::currentTime());
             if (elapsedMsecs > checkpointTime)
                 checkpointTime += 300;
-    
+
             // What we do here is a sigmoidal curve, it starts slowly,
             // then grows more rapidly, slows down again and
             // get asymptotically closer to the maximum.
@@ -624,9 +632,9 @@ bool KDcraw::loadFromDcraw(const QString& filePath, QByteArray &imageData,
         else if (d->dataPos > checkpoint)
         {
             // While receiving data, progress from 40% to 70%
-            double delta = 0.3 + 0.4 - 0.4*part;
+            double delta  = 0.3 + 0.4 - 0.4*part;
             int imageSize = d->width * d->height * (m_rawDecodingSettings.sixteenBitsImage ? 6 : 3);
-            checkpoint += (int)(imageSize / (20 * delta));
+            checkpoint   += (int)(imageSize / (20 * delta));
             setRecievingDataProgress(0.4*part + delta * (((float)d->dataPos)/((float)imageSize))); 
         }
 
@@ -695,16 +703,16 @@ void KDcraw::startProcess()
 
     connect(d->process, SIGNAL(processExited(KProcess *)),
             this, SLOT(slotProcessExited(KProcess *)));
-             
+
     connect(d->process, SIGNAL(receivedStdout(KProcess *, char *, int)),
             this, SLOT(slotReceivedStdout(KProcess *, char *, int)));
-             
+
     connect(d->process, SIGNAL(receivedStderr(KProcess *, char *, int)),
             this, SLOT(slotReceivedStderr(KProcess *, char *, int)));
 
     // run dcraw with options:
     // -c : write to stdout
-    // -v : verboze mode.
+    // -v : verbose mode.
     //
     // -4 : 16bit ppm output
     //
@@ -714,13 +722,17 @@ void KDcraw::startProcess()
     // -n : Use wavelets to erase noise while preserving real detail. 
     // -j : Do not stretch the image to its correct aspect ratio.
     // -q : Use an interpolation method.
-    // -p : Use the input ICC profiles to define the camera's raw colorspace.
-    // -o : Use ICC profiles to define the output colorspace.
     // -h : Output a half-size color image. Twice as fast as -q 0.
     // -b : set Brightness value.
     // -k : set Black Point value.
+    // -S : set White Point value (saturation).
     // -r : set Raw Color Balance Multipliers.
     // -C : set Correct chromatic aberration correction.
+    // -m : After interpolation, clean up color artifacts by repeatedly applying a 3x3 median filter to the R-G and B-G channels.
+    // -A : Calculate the white balance by averaging a rectangular area from image.
+    // -P : Read the dead pixel list from this file.
+    // -p : Use ICC profiles to define the camera's raw colorspace or use embeded profile from raw file.
+    // -o : Use ICC profiles to define the output colorspace.
 
     *d->process << DcrawBinary::path();
     *d->process << "-c";
@@ -741,8 +753,11 @@ void KDcraw::startProcess()
     *d->process << "-H";
     *d->process << QString::number(m_rawDecodingSettings.unclipColors);
 
-    *d->process << "-b";
-    *d->process << QString::number(m_rawDecodingSettings.brightness);
+    if (m_rawDecodingSettings.brightness != 1.0)
+    {
+        *d->process << "-b";
+        *d->process << QString::number(m_rawDecodingSettings.brightness);
+    }
 
     if (m_rawDecodingSettings.enableBlackPoint)
     {
@@ -750,17 +765,42 @@ void KDcraw::startProcess()
         *d->process << QString::number(m_rawDecodingSettings.blackPoint);
     }
 
+    if (m_rawDecodingSettings.enableWhitePoint)
+    {
+        *d->process << "-S";
+        *d->process << QString::number(m_rawDecodingSettings.whitePoint);
+    }
+
+    if (m_rawDecodingSettings.medianFilterPasses > 0)
+    {
+        *d->process << "-m";
+        *d->process << QString::number(m_rawDecodingSettings.medianFilterPasses);
+    }
+
+    if (!m_rawDecodingSettings.deadPixelMap.isEmpty())
+    {
+        *d->process << "-P";
+        *d->process << QFile::encodeName(m_rawDecodingSettings.deadPixelMap);
+    }
+
     switch (m_rawDecodingSettings.whiteBalance)
     {
         case RawDecodingSettings::NONE:
+        {
             break;
+        }
         case RawDecodingSettings::CAMERA:
+        {
             *d->process << "-w";
             break;
+        }
         case RawDecodingSettings::AUTO:
+        {
             *d->process << "-a";
             break;
+        }
         case RawDecodingSettings::CUSTOM:
+        {
             /* Convert between Temperature and RGB.
              */
             double T;
@@ -796,11 +836,11 @@ void KDcraw::startProcess()
             {
                 xD = -2.0064e9/(T*T*T) + 1.9018e6/(T*T) + 0.24748e3/T + 0.237040;
             }
-            yD = -3*xD*xD + 2.87*xD - 0.275;
 
-            X = xD/yD;
-            Y = 1;
-            Z = (1-xD-yD)/yD;
+            yD     = -3*xD*xD + 2.87*xD - 0.275;
+            X      = xD/yD;
+            Y      = 1;
+            Z      = (1-xD-yD)/yD;
             RGB[0] = X*XYZ_to_RGB[0][0] + Y*XYZ_to_RGB[1][0] + Z*XYZ_to_RGB[2][0];
             RGB[1] = X*XYZ_to_RGB[0][1] + Y*XYZ_to_RGB[1][1] + Z*XYZ_to_RGB[2][1];
             RGB[2] = X*XYZ_to_RGB[0][2] + Y*XYZ_to_RGB[1][2] + Z*XYZ_to_RGB[2][2];
@@ -834,6 +874,16 @@ void KDcraw::startProcess()
             *d->process << QString::number(RGB[2], 'f', 5);
             *d->process << QString::number(RGB[1], 'f', 5);
             break;
+        }
+        case RawDecodingSettings::AERA:
+        {
+            *d->process << "-A";
+            *d->process << QString::number(m_rawDecodingSettings.whiteBalanceArea.left());
+            *d->process << QString::number(m_rawDecodingSettings.whiteBalanceArea.top());
+            *d->process << QString::number(m_rawDecodingSettings.whiteBalanceArea.width());
+            *d->process << QString::number(m_rawDecodingSettings.whiteBalanceArea.height());
+            break;
+        }
     }
 
     *d->process << "-q";
@@ -852,8 +902,45 @@ void KDcraw::startProcess()
         *d->process << QString::number(m_rawDecodingSettings.caMultiplier[1], 'f', 5);
     }
 
-    *d->process << "-o";
-    *d->process << QString::number(m_rawDecodingSettings.outputColorSpace);
+    switch (m_rawDecodingSettings.inputColorSpace)
+    {
+        case RawDecodingSettings::EMBEDDED:
+        {
+            *d->process << "-p";
+            *d->process << "embed";
+            break;
+        }
+        case RawDecodingSettings::CUSTOMINPUTCS:
+        {
+            if (!m_rawDecodingSettings.inputProfile.isEmpty())
+            {
+                *d->process << "-p";
+                *d->process << QFile::encodeName(m_rawDecodingSettings.inputProfile);
+            }
+            break;
+        }
+        default:   // No input profile
+            break;
+    }
+
+    switch (m_rawDecodingSettings.outputColorSpace)
+    {
+        case RawDecodingSettings::CUSTOMOUTPUTCS:
+        {
+            if (!m_rawDecodingSettings.outputProfile.isEmpty())
+            {
+                *d->process << "-o";
+                *d->process << QFile::encodeName(m_rawDecodingSettings.outputProfile);
+            }
+            break;
+        }
+        default:
+        {
+            *d->process << "-o";
+            *d->process << QString::number(m_rawDecodingSettings.outputColorSpace);
+            break;
+        }
+    }
 
     *d->process << QFile::encodeName(d->filePath);
 
